@@ -25,7 +25,7 @@ from . import utilities as u
 
 class Catalog:
     """An object to collect SED results for plotting and analysis"""
-    def __init__(self, name='SED Catalog', marker='circle', color='blue', verbose=True,  **kwargs):
+    def __init__(self, name='SED Catalog', marker='circle', color='blue', verbose=True, **kwargs):
         """Initialize the Catalog object"""
         # Metadata
         self.verbose = verbose
@@ -42,8 +42,7 @@ class Catalog:
                      'membership', 'reddening', 'fbol', 'fbol_unc', 'mbol',
                      'mbol_unc', 'Lbol', 'Lbol_unc', 'Lbol_sun',
                      'Lbol_sun_unc', 'Mbol', 'Mbol_unc', 'logg', 'logg_unc',
-                     'mass', 'mass_unc', 'Teff', 'Teff_unc', 'Teff_evo',
-                     'Teff_evo_unc', 'Teff_bb', 'SED']
+                     'mass', 'mass_unc', 'Teff', 'Teff_unc', 'SED']
 
         # A master table of all SED results
         self.results = self.make_results_table(self)
@@ -77,6 +76,48 @@ class Catalog:
 
         return new_cat
 
+    def add_column(self, name, data, unc=None):
+        """
+        Add a column of data to the results table
+
+        Parameters
+        ----------
+        name: str
+            The name of the new column
+        data: sequence
+            The data array
+        unc: sequence (optional)
+            The uncertainty array
+        """
+        # Make sure column doesn't exist
+        if name in self.results.colnames:
+            raise ValueError("{}: Column already exists.".format(name))
+
+        # Make sure data is the right length
+        if len(data) != len(self.results):
+            raise ValueError("{} != {}: Data is not the right size for this catalog.".format(len(data), len(self.results)))
+
+        # Add the column
+        self.results.add_column(data, name=name)
+
+        # Add uncertainty column
+        if unc is not None:
+
+            # Uncertainty name
+            name = name + '_unc'
+
+            # Make sure column doesn't exist
+            if name in self.results.colnames:
+                raise ValueError("{}: Column already exists.".format(name))
+
+            # Make sure data is the right length
+            if len(unc) != len(self.results):
+                raise ValueError(
+                    "{} != {}: Data is not the right size for this catalog.".format(len(unc), len(self.results)))
+
+            # Add the column
+            self.results.add_column(unc, name=name)
+
     def add_SED(self, sed):
         """Add an SED to the catalog
 
@@ -144,8 +185,7 @@ class Catalog:
         # Add the new row
         self.results.add_row(new_row)
 
-        if self.verbose:
-            print("Successfully added SED '{}'".format(sed.name))
+        self.message("Successfully added SED '{}'".format(sed.name))
 
     def export(self, parentdir='.', dirname=None, format='ipac', sources=True, zipped=False):
         """
@@ -176,7 +216,7 @@ class Catalog:
         # Remove '.' from column names
         final = at.Table(self.results).filled(np.nan)
         for col in final.colnames:
-            final.rename_column(col, col.replace('.', '_'))
+            final.rename_column(col, col.replace('.', '_').replace('/', '_'))
 
         # Write a directory of results and all SEDs...
         if sources:
@@ -248,22 +288,26 @@ class Catalog:
         # Get the table of sources
         data = ascii.read(filepath, delimiter=delimiter)
 
-        if self.verbose:
-            print("Generating SEDs for {} sources from {}".format(len(data), filepath))
+        self.message("Generating SEDs for {} sources from {}".format(len(data), filepath))
 
         # Iterate over table
         for row in data:
 
-            # Make the SED
-            s = SED(row['name'], verbose=False)
-            if 'ra' in row and 'dec' in row:
-                s.sky_coords = row['ra']*q.deg, row['dec']*q.deg
+            try:
 
-            # Run the desired methods
-            s.run_methods(run_methods)
+                # Make the SED
+                s = SED(row['name'], verbose=False)
+                if 'ra' in row and 'dec' in row:
+                    s.sky_coords = row['ra']*q.deg, row['dec']*q.deg
 
-            # Add it to the catalog
-            self.add_SED(s)
+                # Run the desired methods
+                s.run_methods(run_methods)
+
+                # Add it to the catalog
+                self.add_SED(s)
+
+            except:
+                self.message("Could not add SED '{}".format(row['name']))
 
     def get_data(self, *args):
         """Fetch the data for the given columns
@@ -311,8 +355,7 @@ class Catalog:
             return copy(self.results[name_or_idx]['SED'])
 
         else:
-            if self.verbose:
-                print('Could not retrieve SED {}'.format(name_or_idx))
+            self.message('Could not retrieve SED {}'.format(name_or_idx))
 
             return
 
@@ -353,16 +396,29 @@ class Catalog:
         results['mass_unc'].unit = q.Msun
         results['Teff'].unit = q.K
         results['Teff_unc'].unit = q.K
-        results['Teff_bb'].unit = q.K
-        results['Teff_evo'].unit = q.K
-        results['Teff_evo_unc'].unit = q.K
 
         return results
 
+    def message(self, msg, pre='[sedkit]'):
+        """
+        Only print message if verbose=True
+
+        Parameters
+        ----------
+        msg: str
+            The message to print
+        pre: str
+            The stuff to print before
+        """
+        if self.verbose:
+            if pre is None:
+                print(msg)
+            else:
+                print("{} {}".format(pre, msg))
+
     def plot(self, x, y, marker=None, color=None, scale=['linear','linear'],
-             xlabel=None, ylabel=None, fig=None, order=None, data=None,
-             identify=None, id_color='red', label_points=False, exclude=None,
-             **kwargs):
+             xlabel=None, ylabel=None, fig=None, order=None, identify=[],
+             id_color='red', label_points=False, draw=True, **kwargs):
         """Plot parameter x versus parameter y
 
         Parameters
@@ -385,8 +441,6 @@ class Catalog:
              The figure to plot on
         order: int
              The polynomial order to fit
-        data: dict
-             Additional data to add to the plot
         identify: idx, str, sequence
              Names of sources to highlight in the plot
         id_color: str
@@ -403,6 +457,12 @@ class Catalog:
         source = copy(self.source)
         params = [k for k in source.column_names if not k.endswith('_unc')]
 
+        # If no uncertainty column for parameter, add it
+        if '{}_unc'.format(x) not in source.column_names:
+            _ = source.add([None] * len(source.data['name']), '{}_unc'.format(x))
+        if '{}_unc'.format(y) not in source.column_names:
+            _ = source.add([None] * len(source.data['name']), '{}_unc'.format(y))
+
         # Check if the x parameter is a color
         if '-' in x and all([i in params for i in x.split('-')]):
             colordata = self.get_data(x)[0]
@@ -437,39 +497,40 @@ class Catalog:
             hover = HoverTool(tooltips=tips, names=['points'])
 
             # Make the plot
-            TOOLS = ['pan', 'reset', 'box_zoom', 'save', hover]
+            TOOLS = ['pan', 'reset', 'box_zoom', 'wheel_zoom', 'save', hover]
             title = '{} v {}'.format(x, y)
             fig = figure(plot_width=800, plot_height=500, title=title, y_axis_type=scale[1], x_axis_type=scale[0], tools=TOOLS)
 
-        # # Exclude sources
-        # if exclude is not None:
-        #     exc_idx = [i for i, v in enumerate(source.data['name']) if v in exclude]
-        #     patches = {x : [(i, np.nan) for i in exc_idx],
-        #                y : [(i, np.nan) for i in exc_idx]}
-        #
-        #     source.patch(patches)
-
         # Get marker class
         size = kwargs.get('size', 8)
         kwargs['size'] = size
         marker = getattr(fig, marker or self.marker)
         color = color or self.color
-        marker(x, y, source=source, color=color, fill_alpha=0.7, name='points', **kwargs)
 
-        # Plot y errorbars
-        yval, yerr = source.data[y], source.data['{}_unc'.format(y)]
-        yval[yval == None] = np.nan
-        yerr[yerr == None] = np.nan
-        y_err_x = [(i, i) for i in source.data[x]]
-        y_err_y = [(i, j) for i, j in zip(yval - yerr, yval + yerr)]
-        fig.multi_line(y_err_x, y_err_y, color=color)
-
-        # Plot x errorbars
+        # Prep data
+        names = source.data['name']
         xval, xerr = source.data[x], source.data['{}_unc'.format(x)]
         xval[xval == None] = np.nan
         xerr[xerr == None] = np.nan
+        yval, yerr = source.data[y], source.data['{}_unc'.format(y)]
+        yval[yval == None] = np.nan
+        yerr[yerr == None] = np.nan
+
+        # Plot nominal values
+        marker(x, y, source=source, color=color, fill_alpha=0.7, name='points', **kwargs)
+
+        # Identify sources
+        idx = [ni for ni, name in enumerate(names) if name in identify]
+        fig.circle(xval[idx], yval[idx], size=size + 5, color=id_color, fill_color=None, line_width=2)
+
+        # Plot y errorbars
+        y_err_x = [(i, i) for i in source.data[x]]
+        y_err_y = [(yval[n] if np.isnan(i - j) else i - j, yval[n] if np.isnan(i + j) else i + j) for n, (i, j) in enumerate(zip(yval, yerr))]
+        fig.multi_line(y_err_x, y_err_y, color=color)
+
+        # Plot x errorbars
         x_err_y = [(i, i) for i in source.data[y]]
-        x_err_x = [(i, j) for i, j in zip(xval - xerr, xval + xerr)]
+        x_err_x = [(xval[n] if np.isnan(i - j) else i - j, xval[n] if np.isnan(i + j) else i + j) for n, (i, j) in enumerate(zip(xval, xerr))]
         fig.multi_line(x_err_x, x_err_y, color=color)
 
         # Label points
@@ -484,7 +545,6 @@ class Catalog:
             idx = [n for n, (i, j) in enumerate(zip(xval, yval)) if not hasattr(i, 'mask') and not np.isnan(i) and not hasattr(j, 'mask') and not np.isnan(j)]
             xd = np.array(xval, dtype=float)[idx]
             yd = np.array(yval, dtype=float)[idx]
-            ye = np.array(yerr, dtype=float)[idx]
 
             # Plot data
             label = 'Order {} fit'.format(order)
@@ -493,53 +553,46 @@ class Catalog:
 
             # Fit the polynomial
             try:
-                coeffs, cov = np.polyfit(x=xd, y=yd, deg=order, w=1./ye, cov=True)
-            except Exception:
-                try:
+
+                if yerr is not None:
+                    ye = np.array(yerr, dtype=float)[idx]
+                    coeffs, cov = np.polyfit(x=xd, y=yd, deg=order, w=1. / ye, cov=True)
+                else:
                     coeffs, cov = np.polyfit(x=xd, y=yd, deg=order, cov=True)
-                except:
-                    pass
 
-            # Plot the line
-            if coeffs is None or any([np.isnan(i) for i in coeffs]):
-                if self.verbose:
-                    print("Could not fit that data with an order {} polynomial".format(order))
-            else:
+                # Plot the line
+                if coeffs is None or any([np.isnan(i) for i in coeffs]):
+                    self.message("Could not fit that data with an order {} polynomial".format(order))
+                else:
 
-                # Calculate values and 1-sigma
-                TT = np.vstack([xaxis**(order-i) for i in range(order+1)]).T
-                yaxis = np.dot(TT, coeffs)
-                C_yi = np.dot(TT, np.dot(cov, TT.T))
-                sig = np.sqrt(np.diag(C_yi))
+                    # Calculate values and 1-sigma
+                    TT = np.vstack([xaxis**(order - i) for i in range(order + 1)]).T
+                    yaxis = np.dot(TT, coeffs)
+                    C_yi = np.dot(TT, np.dot(cov, TT.T))
+                    sig = np.sqrt(np.diag(C_yi))
 
-                # Plot the line and shaded error
-                fig.line(xaxis, yaxis, legend=label, color=color, line_alpha=0.3)
-                xpat = np.hstack((xaxis, xaxis[::-1]))
-                ypat = np.hstack((yaxis+sig, (yaxis-sig)[::-1]))
-                err_source = ColumnDataSource(dict(xaxis=xpat, yaxis=ypat))
-                glyph = Patch(x='xaxis', y='yaxis', fill_color=color, line_color=None, fill_alpha=0.1)
-                fig.add_glyph(err_source, glyph)
+                    # Plot the line and shaded error
+                    fig.line(xaxis, yaxis, legend=label + ' {}'.format(coeffs[::-1]), color=color, line_alpha=0.3)
+                    xpat = np.hstack((xaxis, xaxis[::-1]))
+                    ypat = np.hstack((yaxis + sig, (yaxis - sig)[::-1]))
+                    err_source = ColumnDataSource(dict(xaxis=xpat, yaxis=ypat))
+                    glyph = Patch(x='xaxis', y='yaxis', fill_color=color, line_color=None, fill_alpha=0.1)
+                    fig.add_glyph(err_source, glyph)
+
+            except Exception as exc:
+                print("Skipping the polynomial fit: {}".format(exc))
 
         # Set axis labels
         xunit = source.data[x].unit
         yunit = source.data[y].unit
-        fig.xaxis.axis_label = '{}{}'.format(x, ' [{}]'.format(xunit) if xunit else '')
-        fig.yaxis.axis_label = '{}{}'.format(y, ' [{}]'.format(yunit) if yunit else '')
+        fig.xaxis.axis_label = xlabel or '{}{}'.format(x, ' [{}]'.format(xunit) if xunit else '')
+        fig.yaxis.axis_label = ylabel or '{}{}'.format(y, ' [{}]'.format(yunit) if yunit else '')
 
         # Formatting
         fig.legend.location = "top_right"
 
-        # Identify sources
-        if isinstance(identify, list):
-            id_cat = Catalog('Identified')
-            for obj_id in identify:
-                obj_result = self.get_SED(obj_id)
-                if str(type(obj_result)) != "<class 'astropy.table.column.Column'>":
-                    obj_result = [obj_result]
-                for obj in obj_result:
-                    id_cat.add_SED(obj)
-            fig = id_cat.plot(x, y, fig=fig, size=size+5, marker='circle', line_color=id_color, fill_color=None, line_width=2, label_points=True)
-            del id_cat
+        if draw:
+            show(fig)
 
         return fig
 
@@ -601,8 +654,7 @@ class Catalog:
             self.results.remove_row([name_or_idx])
 
         else:
-            if self.verbose:
-                print('Could not remove SED {}'.format(name_or_idx))
+            self.message('Could not remove SED {}'.format(name_or_idx))
 
             return
 
@@ -627,8 +679,7 @@ class Catalog:
             pickle.dump(self.results, f, pickle.HIGHEST_PROTOCOL)
             f.close()
 
-            if self.verbose:
-                print('Catalog saved to {}'.format(file))
+            self.message('Catalog saved to {}'.format(file))
 
     @property
     def source(self):
