@@ -11,6 +11,7 @@ import glob
 from pkg_resources import resource_filename
 
 import astropy.units as q
+import astropy.constants as ac
 from astropy.io.ascii import read
 from bokeh.plotting import figure, show
 from bokeh.models import LinearColorMapper, BasicTicker, ColorBar
@@ -19,18 +20,21 @@ import numpy as np
 from . import utilities as u
 
 # A dictionary of all supported moving group ages from Bell et al. (2015)
-NYMG_AGES = {'AB Dor': (149*q.Myr, 51*q.Myr),
-             'beta Pic': (24*q.Myr, 3*q.Myr),
-             'Carina': (45*q.Myr, 11*q.Myr),
-             'Columba': (42*q.Myr, 6*q.Myr),
-             'eta Cha': (11*q.Myr, 3*q.Myr),
-             'Tuc-Hor': (45*q.Myr, 4*q.Myr),
-             'TW Hya': (10*q.Myr, 3*q.Myr),
-             '32 Ori': (22*q.Myr, 4*q.Myr)}
+NYMG_AGES = {'AB Dor': (149 * q.Myr, 51 * q.Myr, '2015MNRAS.454..593B'),
+             'beta Pic': (24 * q.Myr, 3 * q.Myr, '2015MNRAS.454..593B'),
+             'Carina': (45 * q.Myr, 11 * q.Myr, '2015MNRAS.454..593B'),
+             'Columba': (42 * q.Myr, 6 * q.Myr, '2015MNRAS.454..593B'),
+             'eta Cha': (11 * q.Myr, 3 * q.Myr, '2015MNRAS.454..593B'),
+             'Tuc-Hor': (45 * q.Myr, 4 * q.Myr, '2015MNRAS.454..593B'),
+             'TW Hya': (10 * q.Myr, 3 * q.Myr, '2015MNRAS.454..593B'),
+             '32 Ori': (22 * q.Myr, 4 * q.Myr, '2015MNRAS.454..593B')}
 
 # A list of all supported evolutionary models
-EVO_MODELS = [os.path.basename(m).replace('.txt', '') for m in glob.glob(resource_filename('sedkit', 'data/models/evolutionary/*'))]
-
+try:
+    EVO_MODELS = [os.path.basename(m).replace('.txt', '') for m in glob.glob(resource_filename('sedkit', 'data/models/evolutionary/*'))]
+# Fails RTD build for some reason
+except:
+    EVO_MODELS = ['COND03', 'dmestar_solar', 'DUSTY00', 'f2_solar_age', 'hybrid_solar_age', 'nc+03_age', 'nc-03_age', 'nc_solar_age', 'parsec12_solar']
 
 class Isochrone:
     """A class to handle model isochrones"""
@@ -66,6 +70,11 @@ class Isochrone:
         if min(self.data['age']) > 100000:
             self.data['age'] *= 1E-9
 
+        # Calculate radii if not in the table (R = sqrt(GM/g))
+        if 'radius' not in self.data.colnames:
+            radius = np.sqrt((ac.G * (self.data['mass'] * q.M_sun)) / ((10**self.data['logg']) * q.m / q.s**2)).to(q.R_sun)
+            self.data.add_column(radius, name='radius')
+
         # Get the units
         if units is None or not isinstance(units, dict):
             units = {'age': q.Gyr, 'mass': q.Msun, 'Lbol': q.Lsun, 'teff': q.K, 'radius': q.Rsun}
@@ -153,9 +162,8 @@ class Isochrone:
 
         # Test the age range is inbounds
         if age[0] < self.ages.min() or age[0] > self.ages.max():
-            if self.verbose:
-                args = age[0], self.ages.min(), self.ages.max(), yparam, self.name
-                print('{}: age must be between {} and {} to infer {} from {} isochrones.'.format(*args))
+            args = age[0], self.ages.min(), self.ages.max(), yparam, self.name
+            self.message('{}: age must be between {} and {} to infer {} from {} isochrones.'.format(*args))
             return None
 
         # Get the lower, nominal, and upper values
@@ -172,7 +180,7 @@ class Isochrone:
         if upper is None:
             return None
 
-        # Caluclate the symmetric error
+        # Calculate the symmetric error
         error = max(abs(nominal - lower), abs(nominal - upper)) * 2
 
         # Plot the figure and evaluated point
@@ -229,9 +237,8 @@ class Isochrone:
         min_x = min(lower_iso[xparam].min(), upper_iso[xparam].min())
         max_x = max(lower_iso[xparam].max(), upper_iso[xparam].max())
         if xval < min_x or xval > max_x:
-            if self.verbose:
-                args = round(xval, 3), xparam, min_x, max_x, yparam, self.name
-                print('{}: {} must be between {} and {} to infer {} from {} isochrones.'.format(*args))
+            args = round(xval, 3), xparam, min_x, max_x, yparam, self.name
+            self.message('{}: {} must be between {} and {} to infer {} from {} isochrones.'.format(*args))
             return None
 
         # Get the neighboring interpolated values
@@ -245,6 +252,23 @@ class Isochrone:
 
         return result * unit
 
+    def message(self, msg, pre='[sedkit.Isochrone]'):
+        """
+        Only print message if verbose=True
+
+        Parameters
+        ----------
+        msg: str
+            The message to print
+        pre: str
+            The stuff to print before
+        """
+        if self.verbose:
+            if pre is None:
+                print(msg)
+            else:
+                print("{} {}".format(pre, msg))
+
     def plot(self, xparam, yparam, draw=False, **kwargs):
         """Plot an evaluated isochrone, isochrone, or set of isochrones
 
