@@ -28,19 +28,43 @@ class TestModelGrid(unittest.TestCase):
 
         self.modelgrid = grid
 
+        self.bt = mg.BTSettl()
+
+    def test_get_spectrum(self):
+        """Test the get_spectrum method"""
+        # On grid specific
+        spec = self.bt.get_spectrum(teff=3500, logg=5.5, alpha=0, meta=0)
+
+        # On grid ambiguous
+        spec = self.bt.get_spectrum(teff=3500)
+
+        # Off grid
+        spec = self.bt.get_spectrum(teff=3456, logg=5.5, alpha=0, meta=0)
+
+    def test_resample_grid(self):
+        """Test resample_grid method"""
+        bt = self.bt
+        new = bt.resample_grid(teff=[3500, 3600, 3700], logg=[5.5], meta=[0], alpha=[0])
+        self.assertTrue(len(bt.index) > len(new.index))
+
     def test_filter(self):
         """Test the filter metod works"""
-        filt = self.modelgrid.filter(SpT=70)
+        filt = self.bt.filter(teff=3500)
         self.assertEqual(len(filt), 1)
 
+    def test_photometry(self):
+        """Test the photometry method"""
+        phot_mg = self.bt.photometry(['2MASS.J', '2MASS.H', '2MASS.Ks'])
+        self.assertEqual(phot_mg.index.iloc[0]['spectrum'][0].size, 3)
+
     def test_plot(self):
         """Test that the plot method works"""
-        plt = self.modelgrid.plot(SpT=70, draw=False)
+        plt = self.bt.plot(teff=3500, draw=False)
         self.assertEqual(str(type(plt)), "<class 'bokeh.plotting.figure.Figure'>")
 
     def test_save(self):
         """Test the save method works"""
-        self.modelgrid.save('test.p')
+        self.bt.save('test.p')
         os.system('rm test.p')
 
 
@@ -57,11 +81,12 @@ def test_load_model():
 
 def test_load_ModelGrid():
     """Test the load_ModelGrid function"""
-    path = 'data/models/atmospheric/Filippazzo2016.p'
-    filepath = resource_filename('sedkit', path)
-
-    lmg = mg.load_ModelGrid(filepath)
+    grid = mg.BTSettl()
+    path = './test.p'
+    grid.save(path)
+    lmg = mg.load_ModelGrid(path)
     assert isinstance(lmg, mg.ModelGrid)
+    os.system('rm test.p')
 
 
 def test_BTSettl():
@@ -74,9 +99,3 @@ def test_SpexPrismLibrary():
     """Test the SpexPrismLibrary grid"""
     grid = mg.SpexPrismLibrary()
     assert hasattr(grid, 'name')
-
-
-def test_Filippazzo2016():
-    """Test the Filippazzo2016 grid"""
-    grid = mg.Filippazzo2016()
-    assert hasattr(grid, 'name')
