@@ -5,6 +5,7 @@ from pkg_resources import resource_filename
 import numpy as np
 import astropy.units as q
 from astropy.modeling.blackbody import blackbody_lambda
+from astropy.coordinates import SkyCoord
 
 from .. import sed
 from .. import spectrum as sp
@@ -27,7 +28,7 @@ class TestSED(unittest.TestCase):
         SPEC2 = [WAVE2, FLUX2, FLUX2/100.]
         self.spec2 = sp.Spectrum(*SPEC2)
 
-        self.sed = sed.SED(verbose=True)
+        self.sed = sed.SED(verbose=True, foo=123)
 
     def test_add_photometry(self):
         """Test that photometry is added properly"""
@@ -60,8 +61,10 @@ class TestSED(unittest.TestCase):
 
         # Make sure the units are being updated
         self.assertEqual(len(s.spectra), 2)
-        self.assertEqual(s.spectra[0]['spectrum'].wave_units,
-                         s.spectra[1]['spectrum'].wave_units)
+        self.assertEqual(s.spectra[0]['spectrum'].wave_units, s.spectra[1]['spectrum'].wave_units)
+
+        # Call results to test group_spectra method
+        s.results
 
         # Test removal
         s.drop_spectrum(0)
@@ -83,44 +86,54 @@ class TestSED(unittest.TestCase):
         s.name = b'Gl 752B'
 
         # Age
-        s.age = 4*q.Gyr, 0.1*q.Gyr
+        s.age = 4 * q.Gyr, 0.1 * q.Gyr
+        s.age = 4 * q.Gyr, 0.1 * q.Gyr, 'reference'
         self.assertRaises(TypeError, setattr, s, 'age', 'foo')
         self.assertRaises(TypeError, setattr, s, 'age', (4, 0.1))
         self.assertRaises(TypeError, setattr, s, 'age', (4*q.Jy, 0.1*q.Jy))
 
         # Dec
-        s.dec = 1.2345*q.deg
+        s.dec = 1.2345 * q.deg
         self.assertRaises(TypeError, setattr, s, 'dec', 1.2345)
 
         # RA
-        s.ra = 1.2345*q.deg
+        s.ra = 1.2345 * q.deg
         self.assertRaises(TypeError, setattr, s, 'ra', 1.2345)
 
+        # Sky coords
+        s.sky_coords = 1.2345 * q.deg, 1.2345 * q.deg
+        s.sky_coords = '1.2345', '1.2345'
+        self.assertRaises(TypeError, setattr, s, 'sky_coords', 'foo')
+        self.assertRaises(TypeError, setattr, s, 'sky_coords', None)
+
         # Distance
         s.distance = None
-        s.distance = 1.2*q.pc, 0.1*q.pc
+        s.distance = 1.2 * q.pc, 0.1 * q.pc
+        s.distance = 1.2 * q.pc, 0.1 * q.pc, 'reference'
         self.assertRaises(TypeError, setattr, s, 'distance', (1, 2, 3, 4))
         self.assertRaises(TypeError, setattr, s, 'distance', (1, 4))
 
         # Parallax
         s.parallax = None
-        s.parallax = 1.2*q.mas, 0.1*q.mas
+        s.parallax = 1.2 * q.mas, 0.1 * q.mas
+        s.parallax = 1.2 * q.mas, 0.1 * q.mas, 'reference'
         self.assertRaises(TypeError, setattr, s, 'parallax', (1, 2, 3, 4))
         self.assertRaises(TypeError, setattr, s, 'parallax', (1, 4))
 
         # Radius
         s.radius = None
-        s.radius = 1.2*q.R_jup, 0.1*q.R_jup
+        s.radius = 1.2 * q.R_jup, 0.1 * q.R_jup
+        s.radius = 1.2 * q.R_jup, 0.1 * q.R_jup, 'reference'
         self.assertRaises(TypeError, setattr, s, 'radius', (1, 2, 3, 4))
         self.assertRaises(TypeError, setattr, s, 'radius', (1, 4))
 
         # Spectral type
-        s.spectral_type = 'M3'
+        s.spectral_type = 'M3V', 'reference'
         s.radius = None
-        s.spectral_type = 20
+        s.spectral_type = 20, 0.5
         s.age = None
-        s.spectral_type = 10, 1, 'b', 'V', 'sd'
-        self.assertRaises(TypeError, setattr, s, 'spectral_type', ['foo'])
+        s.spectral_type = [10, 1, 'b', 'V', 'sd'], 'reference'
+        self.assertRaises(ValueError, setattr, s, 'spectral_type', ['foo'])
 
         # Evolutionary Model
         s.evo_model = 'COND03'
@@ -162,20 +175,22 @@ class TestSED(unittest.TestCase):
         s.make_wein_tail(teff=2000*q.K)
 
         # Radius from spectral type
-        s.radius_from_spectral_type('foo')
-        s.radius_from_spectral_type()
+        s.results
 
-        # Radius from age
-        s.radius_from_age()
+    def test_compare_model(self):
+        """Test for the compare_model method"""
+        v = sed.VegaSED()
+        bt = mg.BTSettl()
+        v.compare_model(bt, teff=3000)
 
     def test_plot(self):
         """Test plotting method"""
-        s = copy.copy(self.sed)
-        f = resource_filename('sedkit', 'data/L3_photometry.txt')
-        s.add_photometry_file(f)
-        s.make_sed()
-
-        fig = s.plot(integral=True)
+        v = sed.VegaSED()
+        v.calculate_synthetic_photometry()
+        bt = mg.BTSettl()
+        v.fit_modelgrid(bt)
+        v.results
+        fig = v.plot(integral=True, synthetic_photometry=True, best_fit=True)
 
     def test_no_photometry(self):
         """Test that a purely photometric SED can be creted"""
@@ -211,20 +226,58 @@ class TestSED(unittest.TestCase):
         s.find_Gaia()
 
         self.assertNotEqual(len(s.photometry), 0)
+        self.assertTrue(isinstance(s.get_mag('2MASS.J'), tuple))
 
-    def test_fit_spectrum(self):
-        """Test that the SED can be fit by a model grid"""
+    def test_find_SDSS_spectra(self):
+        """Test the find_SDSS_spectra method"""
+        s = sed.SED()
+        s.sky_coords = SkyCoord('0h8m05.63s +14d50m23.3s', frame='icrs')
+        s.find_SDSS_spectra(search_radius=20 * q.arcsec)
+        assert len(s.spectra) > 0
+
+    def test_run_methods(self):
+        """Test that the method_list argument works"""
+        s = sed.SED('trappist-1', method_list=['find_2MASS'])
+
+        self.assertNotEqual(len(s.photometry), 0)
+
+    def test_synthetic_photometry(self):
+        """Test the calculate_synthetic_photometry method"""
+        v = sed.VegaSED()
+        v.calculate_synthetic_photometry()
+
+        self.assertTrue(len(v.synthetic_photometry) > 0)
+
+    def test_fit_spectral_type(self):
+        """Test that the SED can be fit by a spectral type atlas"""
         # Grab the SPL
-        spl = mg.SpexPrismLibrary()
+        bt = mg.BTSettl()
 
         # Add known spectrum
         s = copy.copy(self.sed)
-        spec = spl.get_spectrum(snr=100)
+        spec = bt.get_spectrum(snr=100)
         s.add_spectrum(spec)
 
         # Fit with SPL
         s.fit_spectral_type()
 
+    def test_fit_modelgrid(self):
+        """Test that the SED can be fit by a model grid"""
+        # Grab BTSettl
+        bt = mg.BTSettl()
+
+        # Add known spectrum
+        s = copy.copy(self.sed)
+        spec = bt.get_spectrum(snr=100)
+        spec = spec.resamp(wave=np.linspace(0.4, 3, 200) * q.um)
+        s.add_spectrum(spec)
+
+        # Find best grid point
+        s.fit_modelgrid(bt)
+
+        # Fit with mcmc
+        s.fit_modelgrid(bt, mcmc=True)
+
     def test_fit_blackbody(self):
         """Test that the SED can be fit by a blackbody"""
         # Grab the SPL
